@@ -30,16 +30,6 @@
 #include "SimpleCommand.hh"
 #include "I18n.hh"
 
-//use GNU extensions
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif // _GNU_SOURCE
-
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif // HAVE_SYS_TYPES_H
@@ -111,14 +101,14 @@ void allocateUnallocatedColors(std::vector<XColor> colors, Display* dpy, Colorma
     // 'icolors' will hold the first 'nr_icolors' colors of the
     // given (indexed) colormap.
     const size_t nr_icolors = std::min(256, 1 << screen_depth);
-    XColor icolors[nr_icolors];
+    std::vector<XColor> icolors(nr_icolors);
 
     // give each icolor an index
     for (i = 0; i < nr_icolors; i++)
         icolors[i].pixel = i;
 
     // query the colors of the colormap and store them into 'icolors'
-    XQueryColors(dpy, cmap, icolors, nr_icolors);
+    XQueryColors(dpy, cmap, &icolors[0], nr_icolors);
 
     // try to find a close color for all not allocated colors
     for (i = 0; i < colors.size(); i++) {
@@ -188,7 +178,7 @@ ImageControl::ImageControl(int screen_num,
     cache_max = cmax;
 
     if (cache_timeout && s_timed_cache) {
-        m_timer.setTimeout(cache_timeout);
+        m_timer.setTimeout(cache_timeout * FbTk::FbTime::IN_MILLISECONDS);
         RefCount<Command<void> > clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
         m_timer.setCommand(clean_cache);
         m_timer.start();
@@ -495,7 +485,7 @@ void ImageControl::createColorTable() {
         }
 
         if (m_colors_per_channel < 2 || num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
-            fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %ld "
+            fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %zd "
                     "(%d/%d/%d) - reducing",
                     num_colors, m_colors_per_channel, m_colors_per_channel,
                     m_colors_per_channel);
@@ -543,7 +533,7 @@ void ImageControl::createColorTable() {
             }
 
             if (m_colors_per_channel < 2 || num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
-                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %ld "
+                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %zd "
                         "(%d/%d/%d) - reducing",
                         num_colors, m_colors_per_channel, m_colors_per_channel,
                         m_colors_per_channel);
