@@ -142,28 +142,24 @@ void IconButton::setPixmap(bool use) {
 
 void IconButton::reconfigTheme() {
 
-    if (m_theme->texture().usePixmap())
+    setFont(m_theme->text().font());
+    setGC(m_theme->text().textGC());
+    setBorderWidth(m_theme->border().width());
+    setBorderColor(m_theme->border().color());
+    setJustify(m_theme->text().justify());
+    setAlpha(parent()->alpha());
+
+    if (m_theme->texture().usePixmap()) {
         m_pm.reset(m_win.screen().imageControl().renderImage(
                            width(), height(), m_theme->texture(),
                            orientation()));
-    else
-        m_pm.reset(0);
-
-    setAlpha(parent()->alpha());
-
-    if (m_pm != 0)
         setBackgroundPixmap(m_pm);
-    else
+    } else{
+        m_pm.reset(0);
         setBackgroundColor(m_theme->texture().color());
-
-    setGC(m_theme->text().textGC());
-    setFont(m_theme->text().font());
-    setJustify(m_theme->text().justify());
-    setBorderWidth(m_theme->border().width());
-    setBorderColor(m_theme->border().color());
+    }
 
     updateBackground(false);
-
 }
 
 void IconButton::reconfigAndClear() {
@@ -254,6 +250,7 @@ void IconButton::setupWindow() {
 }
 
 void IconButton::drawText(int x, int y, FbTk::FbDrawable *drawable) {
+
     // offset text
     if (m_icon_pixmap.drawable() != 0)
         FbTk::TextButton::drawText(m_icon_window.x() + m_icon_window.width() + 1, y, drawable);
@@ -273,8 +270,7 @@ bool IconButton::setOrientation(FbTk::Orientation orient) {
         FbTk::translatePosition(orient, iconx, icony, m_icon_window.width(), m_icon_window.height(), 0);
         m_icon_window.move(iconx, icony);
         return true;
-    } else {
-        return false;
     }
+    return false;
 }
 
