@@ -25,7 +25,6 @@
 #include "FbTk/ImageControl.hh"
 #include "FbTk/TextUtils.hh"
 #include "FbTk/MemFun.hh"
-#include "FbTk/SimpleObserver.hh"
 
 #include "AtomHandler.hh"
 #include "fluxbox.hh"
@@ -76,18 +75,15 @@ public:
         int actual_format;
         unsigned long nitems, bytes_after;
         unsigned long *prop;
-        bool mapped = false;
         Atom embed_info = SystemTray::getXEmbedInfoAtom();
         if (property(embed_info, 0l, 2l, false, embed_info,
                      &actual_type, &actual_format, &nitems, &bytes_after,
                      (unsigned char **) &prop) && prop != 0) {
-            mapped = (bool)(static_cast<unsigned long>(prop[1]) & XEMBED_MAPPED);
+
             XFree(static_cast<void *>(prop));
-
-
-            fbdbg<<"(SystemTray::TrayWindow::getMappedDefault(): XEMBED_MAPPED = "<<mapped<<endl;
-
-
+            fbdbg << "(SystemTray::TrayWindow::getMappedDefault(): XEMBED_MAPPED = "
+                << (bool)(static_cast<unsigned long>(prop[1]) & XEMBED_MAPPED)
+                << endl;
         }
         return true;
     }
@@ -167,11 +163,10 @@ SystemTray::SystemTray(const FbTk::FbWindow& parent,
     FbTk::EventManager::instance()->add(*this, m_window);
     FbTk::EventManager::instance()->add(*this, m_selection_owner);
     // setup signals
-    m_observer.reset(makeObserver(*this, &SystemTray::update));
-    m_theme->reconfigSig().attach(m_observer.get());
+    join(m_theme->reconfigSig(), FbTk::MemFun(*this, &SystemTray::update));
 
     join(screen.bgChangeSig(),
-         FbTk::MemFun(*this, &SystemTray::updateForScreen));
+         FbTk::MemFunIgnoreArgs(*this, &SystemTray::update));
 
 
     Fluxbox* fluxbox = Fluxbox::instance();
@@ -248,7 +243,7 @@ void SystemTray::resize(unsigned int width, unsigned int height) {
         m_window.resize(width, height);
         if (m_num_visible_clients)
             rearrangeClients();
-        resizeSig().notify();
+        resizeSig().emit();
     }
 }
 
@@ -259,7 +254,7 @@ void SystemTray::moveResize(int x, int y,
         m_window.moveResize(x, y, width, height);
         if (m_num_visible_clients)
             rearrangeClients();
-        resizeSig().notify();
+        resizeSig().emit();
     } else {
         move(x, y);
     }
@@ -437,7 +432,7 @@ void SystemTray::handleEvent(XEvent &event) {
                 (*it)->sendConfigureNotify(0, 0, (*it)->width(), (*it)->height());
                 // so toolbar know that we changed size
                 // done inside this loop, because otherwise we can get into nasty looping
-                resizeSig().notify();
+                resizeSig().emit();
             }
         }
 
