@@ -55,7 +55,7 @@ FbTk::Command<void> *WindowListCmd::parse(const string &command, const string &a
     FbTk::Command<void> *cmd = 0;
     FbTk::Command<bool> *filter = 0;
     std::vector<string> tokens;
-    int opts;
+    int opts = 0;
     string pat;
 
     FbTk::StringUtil::stringTokensBetween(tokens, args, pat, '{', '}');
@@ -86,12 +86,15 @@ void WindowListCmd::execute() {
         // save old value, so we can restore it later
         WinClient *old = WindowCmd<void>::client();
         for (; it != it_end; ++it) {
-            if (typeid(**it) == typeid(FluxboxWindow))
-                WindowCmd<void>::setWindow((*it)->fbwindow());
-            else if (typeid(**it) == typeid(WinClient))
-                WindowCmd<void>::setClient(dynamic_cast<WinClient *>(*it));
-            if (!m_filter || m_filter->execute())
+            Focusable* wptr = *it;
+            if (typeid(*wptr) == typeid(FluxboxWindow)) {
+                WindowCmd<void>::setWindow((wptr)->fbwindow());
+            } else if (typeid(*wptr) == typeid(WinClient)) {
+                WindowCmd<void>::setClient(dynamic_cast<WinClient *>(wptr));
+            }
+            if (!m_filter || m_filter->execute()) {
                 m_cmd->execute();
+            }
         }
         WindowCmd<void>::setClient(old);
     }
@@ -186,7 +189,20 @@ FbTk::Command<void> *parseWindowList(const string &command,
     } else if (command == "arrangewindowshorizontal") {
         int method = ArrangeWindowsCmd::HORIZONTAL;
         return new ArrangeWindowsCmd(method,pat);
+     } else if (command == "arrangewindowsstackleft") {
+        int method = ArrangeWindowsCmd::STACKLEFT;
+        return new ArrangeWindowsCmd(method,pat);
+    } else if (command == "arrangewindowsstackright") {
+        int method = ArrangeWindowsCmd::STACKRIGHT;
+        return new ArrangeWindowsCmd(method,pat);
+    } else if (command == "arrangewindowsstacktop") {
+        int method = ArrangeWindowsCmd::STACKTOP;
+        return new ArrangeWindowsCmd(method,pat);
+    } else if (command == "arrangewindowsstackbottom") {
+        int method = ArrangeWindowsCmd::STACKBOTTOM;
+        return new ArrangeWindowsCmd(method,pat);
     }
+
     return 0;
 }
 
@@ -198,6 +214,10 @@ REGISTER_COMMAND_PARSER(prevgroup, parseWindowList, void);
 REGISTER_COMMAND_PARSER(arrangewindows, parseWindowList, void);
 REGISTER_COMMAND_PARSER(arrangewindowsvertical, parseWindowList, void);
 REGISTER_COMMAND_PARSER(arrangewindowshorizontal, parseWindowList, void);
+REGISTER_COMMAND_PARSER(arrangewindowsstackleft, parseWindowList, void);
+REGISTER_COMMAND_PARSER(arrangewindowsstackright, parseWindowList, void);
+REGISTER_COMMAND_PARSER(arrangewindowsstacktop, parseWindowList, void);
+REGISTER_COMMAND_PARSER(arrangewindowsstackbottom, parseWindowList, void);
 
 } // end anonymous namespace
 
@@ -217,7 +237,7 @@ void AttachCmd::execute() {
                     first->attachClient((*it)->fbwindow()->winClient());
             }
         }
-                
+
     }
 }
 
@@ -390,27 +410,56 @@ void ArrangeWindowsCmd::execute() {
     //        -  only on current head
     const int head = screen->getCurrHead();
     Workspace::Windows::iterator win;
-
     Workspace::Windows normal_windows;
     Workspace::Windows shaded_windows;
-    for(win = space->windowList().begin(); win != space->windowList().end(); win++) {
+    FluxboxWindow* main_window = NULL; // Main (big) window for stacked modes
+
+    for (win = space->windowList().begin(); win != space->windowList().end(); ++win) {
         int winhead = screen->getHead((*win)->fbWindow());
         if ((winhead == head || winhead == 0) && m_pat.match(**win)) {
-            if ((*win)->isShaded())
-                shaded_windows.push_back(*win);
-            else
-                normal_windows.push_back(*win);
+
+            if ((m_tile_method >= STACKLEFT) && (*win)->isFocused()) {
+                main_window = (*win);
+            } else {
+                if ((*win)->isShaded())
+                    shaded_windows.push_back(*win);
+                else
+                    normal_windows.push_back(*win);
+            }
         }
     }
 
+    // if using stacked-left/right/top/bottom and we don't have a main window yet
+    // (no focused window?), we'll fall back on using the last window in the
+    // window list.
+    if (main_window == NULL && (m_tile_method >= STACKLEFT)) {
+        main_window = normal_windows.back();
+        normal_windows.pop_back();
+    }
+
     // to arrange only shaded windows is a bit pointless imho (mathias)
     size_t win_count = normal_windows.size();
-    if (win_count == 0)
-        return;
+    if (win_count == 0) {
+        if (!main_window) {
+            return;
+        }
+        win_count = 1;
+    }
 
-    const unsigned int max_width = screen->maxRight(head) - screen->maxLeft(head);
+
+    int x_offs = screen->maxLeft(head); // window position offset in x
+    int y_offs = screen->maxTop(head); // window position offset in y
+    unsigned int max_width = screen->maxRight(head) - screen->maxLeft(head);
     unsigned int max_height = screen->maxBottom(head) - screen->maxTop(head);
 
+    if ((m_tile_method == STACKLEFT) || (m_tile_method == STACKRIGHT)) {
+        max_width = max_width / 2;
+    }
+
+    if ((m_tile_method == STACKTOP) || (m_tile_method == STACKBOTTOM)) {
+        max_height = max_height / 2;
+    }
+
     // try to get the same number of rows as columns.
     unsigned int cols = int(sqrt((float)win_count));  // truncate to lower
     unsigned int rows = int(0.99 + float(win_count) / float(cols));
@@ -419,10 +468,27 @@ void ArrangeWindowsCmd::execute() {
         std::swap(cols, rows);
     }
 
-    unsigned int x_offs = screen->maxLeft(head); // window position offset in x
-    unsigned int y_offs = screen->maxTop(head); // window position offset in y
-   // unsigned int window = 0; // current window
-    const unsigned int cal_width = max_width/cols; // calculated width ratio (width of every window)
+    // Stacked mode only uses half the screen for tiled windows, so adjust
+    // offset to half the screen (horizontal or vertical depending on 
+    // stacking mode)
+    switch (m_tile_method) {
+    case STACKRIGHT:
+        x_offs += static_cast<int>(max_width);
+        break;
+    case STACKBOTTOM:
+        y_offs += static_cast<int>(max_height);
+        break;
+    default:
+        // no change needed for STACKLEFT/STACKTOP
+        break;
+    }
+
+    // Since the placing algorithm loop below modifies the offsets, but we
+    // still need them to position the main window, we save the calculated
+    // values.
+    const int orig_x_offs = x_offs;
+    const int orig_y_offs = y_offs;
+
     unsigned int i;
     unsigned int j;
 
@@ -430,7 +496,7 @@ void ArrangeWindowsCmd::execute() {
     // TODO: until i resolve the shadedwindow->moveResize() issue to place
     // them in the same columns as the normal windows i just place the shaded
     // windows unchanged ontop of the current head
-    for (i = 0, win = shaded_windows.begin(); win != shaded_windows.end(); win++, i++) {
+    for (i = 0, win = shaded_windows.begin(); win != shaded_windows.end(); ++win, ++i) {
         if (i & 1)
             (*win)->move(x_offs, y_offs);
         else
@@ -445,11 +511,13 @@ void ArrangeWindowsCmd::execute() {
     if (!shaded_windows.empty())
         max_height -= i * (*shaded_windows.begin())->frame().height();
 
+    const unsigned int cal_width = max_width/cols; // width ratio (width of every window)
     const unsigned int cal_height = max_height/rows; // height ratio (height of every window)
+
     // Resizes and sets windows positions in columns and rows.
     for (i = 0; i < rows; ++i) {
-        x_offs = screen->maxLeft(head);
-        for (j = 0; j < cols && normal_windows.size() > 0; ++j) {
+        x_offs = orig_x_offs;
+        for (j = 0; j < cols && !normal_windows.empty(); ++j) {
 
 
             int cell_center_x = x_offs + (x_offs + cal_width) / 2;
@@ -457,11 +525,11 @@ void ArrangeWindowsCmd::execute() {
             unsigned int closest_dist = ~0;
 
             Workspace::Windows::iterator closest = normal_windows.end();
-            for (win = normal_windows.begin(); win != normal_windows.end(); win++) {
+            for (win = normal_windows.begin(); win != normal_windows.end(); ++win) {
 
                 int win_center_x = (*win)->frame().x() + ((*win)->frame().x() + (*win)->frame().width() / 2);
                 int win_center_y = (*win)->frame().y() + ((*win)->frame().y() + (*win)->frame().height() / 2);
-                unsigned int dist = (win_center_x - cell_center_x) * (win_center_x - cell_center_x) + 
+                unsigned int dist = (win_center_x - cell_center_x) * (win_center_x - cell_center_x) +
                                     (win_center_y - cell_center_y) * (win_center_y - cell_center_y);
 
                 if (dist < closest_dist) {
@@ -470,25 +538,51 @@ void ArrangeWindowsCmd::execute() {
                 }
             }
 
-            if (normal_windows.size() > 1) {
-                (*closest)->moveResize(x_offs + (*closest)->xOffset(),
-			y_offs + (*closest)->yOffset(),
-			cal_width - (*closest)->widthOffset(),
-			cal_height - (*closest)->heightOffset());
-            } else { // the last window gets everything that is left.
-                (*closest)->moveResize(x_offs + (*closest)->xOffset(),
-			y_offs + (*closest)->yOffset(),
-			screen->maxRight(head) - x_offs - (*closest)->widthOffset(),
-			cal_height - (*closest)->heightOffset());
+            int x = x_offs + (*closest)->xOffset();
+            int y = y_offs + (*closest)->yOffset();
+            unsigned int w = cal_width - (*closest)->widthOffset();
+            unsigned int h = cal_height - (*closest)->heightOffset();
+
+            // the last window gets everything that is left.
+            if (normal_windows.size() == 1) {
+
+                w = static_cast<int>(screen->maxRight(head)) - x_offs - (*closest)->widthOffset();
+                h = static_cast<int>(cal_height) - (*closest)->heightOffset();
+
+                if (m_tile_method == STACKLEFT) {
+                    w -= max_width;
+                }
             }
 
+            (*closest)->moveResize(x, y, w, h);
             normal_windows.erase(closest);
 
-            // next x offset
-            x_offs += cal_width;
+            x_offs += static_cast<int>(cal_width);
+        }
+
+        y_offs += static_cast<int>(cal_height);
+    }
+
+    // If using a stacked mechanism we now need to place the main window.
+    if (main_window != NULL){
+        x_offs = screen->maxLeft(head);
+        switch (m_tile_method){
+            case STACKLEFT:
+                main_window->moveResize(x_offs + max_width, orig_y_offs, max_width, max_height);
+                break;
+            case STACKRIGHT:
+                main_window->moveResize(x_offs, screen->maxTop(head), max_width, max_height);
+                break;
+            case STACKTOP:
+                main_window->moveResize(x_offs, max_height, max_width, max_height);
+                break;
+            case STACKBOTTOM:
+                main_window->moveResize(x_offs, screen->maxTop(head), max_width, max_height);
+                break;
+            default:
+                // Shouldn't happen.
+                break;
         }
-        // next y offset
-        y_offs += cal_height;
     }
 }
 
@@ -519,7 +613,7 @@ void ShowDesktopCmd::execute() {
         BScreen::Icons icon_list = screen->iconList();
         BScreen::Icons::reverse_iterator iconit = icon_list.rbegin();
         BScreen::Icons::reverse_iterator itend= icon_list.rend();
-        for(; iconit != itend; iconit++) {
+        for(; iconit != itend; ++iconit) {
             if ((*iconit)->workspaceNumber() == space || (*iconit)->isStuck())
                 (*iconit)->deiconify(false);
         }
